#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Define the RISC primary module."""
import hashlib
import logging
import os
from typing import Any, Dict, List

import fire
import requests
from requests.models import Response
from requests.sessions import Session

from .models import (
    RiscAssessment,
    RiscAssessments,
    RiscDeviceConnectivityParent,
    RiscStackConnectivityParent,
)
from .utils import format_bytes, get_user_agent, handle_disk_sizing

logger = logging.getLogger(__name__)


class RISC:
    """Define the RISC toplevel class."""

    def __init__(
        self, api_token: str = "", user_id: str = "", password: str = ""
    ) -> None:
        """Initialize the RISC class."""
        self.api_host: str = os.environ.get(
            "RISC_API_HOST", "https://api.riscnetworks.com"
        )
        self.api_version: str = os.environ.get("RISC_API_VERSION", "1_0")
        self.api_endpoint: str = os.environ.get(
            "RISC_API_ENDPOINT", f"{self.api_host}/{self.api_version}"
        )
        self.assessment_code: str = os.environ.get("RISC_ASSESSMENT_CODE", "")
        self.assessment_filters: Dict[str, str] = dict(
            zip(*[iter(os.environ.get("RISC_ASSESSMENT_FILTERS", "").split(","))] * 2)
        )

        self.auth: Dict[str, str] = {
            "user_id": user_id or os.environ.get("RISC_USERNAME", ""),
            "password": password or os.environ.get("RISC_PASSWORD", ""),
            "api_token": api_token or os.environ.get("RISC_API_TOKEN", ""),
        }
        self.session: Session = requests.Session()
        self.session.headers.update({"User-Agent": get_user_agent()})

        if self.assessment_filters:
            self.assessment = self.get_assessment(**self.assessment_filters)
            self.assessment_code = self.assessment.assessment_code

        if (
            not self.assessment_code
            and not self.assessment_filters
            or (self.assessment_filters and not self.assessment_code)
        ):
            logger.error("You must configure the assessment code or filter criteria!")
            return

        self.token = self.get_auth_token()
        self.session.headers.update(
            {"token": self.token, "assessmentcode": self.assessment_code}
        )

        try:
            self.stack_data = self.stacks_get_summary().json()
        except Exception as e:
            logger.error(
                "Error encountered while attempting to fetch RISC stack data! Error: (%s)"
                % e
            )

    def __repr__(self):
        """Provide the representation for the RISC object."""
        if self.assessment:
            return f"<RISC - User: {self.auth.get('user_id', '')} - Assessment: {self.assessment.assessment_code}>"
        return f"<RISC - User: {self.auth.get('user_id', '')}>"

    def build_auth(self) -> Dict[str, str]:
        """Build the API authentication token."""
        user_id: str = self.auth.get("user_id", "")
        _api_token: str = self.auth.get("api_token", "")
        _password: str = self.auth.get("password", "")
        logger.info("Building authentication dictionary for user: %s" % user_id)

        md5_password: str = hashlib.md5(_password.encode()).hexdigest().upper()
        md5_api_token: str = f"{_api_token}{md5_password}"
        auth_string: str = hashlib.md5(md5_api_token.encode()).hexdigest()
        logger.info("Authentication md5 hash: %s" % auth_string)
        auth = {"userid": user_id, "password": auth_string}

        if self.assessment_code:
            auth["assessmentcode"] = self.assessment_code
        return auth

    def get_assessments(self) -> RiscAssessments:
        """Get the RISC assessment code.

        Returns:
            str: The RISC assessment code.

        """
        payload = self.build_auth()
        response: Response = self.session.get(
            f"{self.api_endpoint}/getAssessments", headers=payload
        )

        if response.status_code != 200:
            logger.error("Unable to retrieve the assessment code!")
            return RiscAssessments()

        assessment_items = response.json().get("assessments", [])
        _assessments = RiscAssessments(assessments=assessment_items)
        return _assessments

    def get_assessment(self, **kwargs) -> RiscAssessment:
        """Get the RISC assessment code.

        Returns:
            str: The RISC assessment code.

        """
        response_data: RiscAssessments = self.get_assessments()
        if not response_data:
            return RiscAssessment()

        if len(response_data.assessments) > 1:
            logger.warn(
                "Multiple assessments found for the provided filter criteria! Returning the first result..."
            )
        return next(item for item in response_data.assessments if not item.is_demo)

    def get_auth_token(self):
        """Authenticate with RISC.

        Returns:
            str: The final authentication token to be used with subsequent requests.

        """
        token: str = ""
        payload: Dict[str, str] = self.build_auth()
        response: Response = self.session.post(
            f"{self.api_endpoint}/getAuthToken", json=payload
        )

        if response.status_code != 200:
            logger.info("Unable to get the authentication token!")
            return token

        try:
            token = response.json().get("token", "")
        except Exception as e:
            logger.error(
                "Error encountered while fetching the authentication token: %s" % e
            )
        return token

    def assets_get_summary(self):
        """Use to retrieve a list of device types and counts."""
        response: Response = self.session.get(f"{self.api_endpoint}/assets/getSummary")
        return response

    def lookup_stack_id(self, name: str):
        if not self.stack_data:
            logger.error("Stack data unavailable!")
            return {}
        try:
            stack_id = next(
                item for item in self.stack_data["assets"] if item["stack_name"] == name
            )["stackid"]
        except Exception as e:
            print(f"No stack_id found for: {name} - Exception: {e}")
            return ""
        return stack_id

    def assets_get_assets(
        self,
        device_type: str = "",
        stack_id: int = 0,
        device_id: str = "",
        tag_id: str = "",
        page: int = 0,
    ) -> Response:
        """Use to retrieve a list of device types and counts."""
        uri_base = (
            f"{self.api_endpoint}/assets/getAssets{'/paginated/' if page else ''}"
        )

        if page:
            self.session.headers.update({"page": str(page)})

        if device_type:
            uri = f"{uri_base}/byType/{device_type}"
        elif stack_id:
            uri = f"{uri_base}/byStack/{stack_id}"
        elif device_id:
            uri = f"{uri_base}/byDevice/{device_id}"
        elif tag_id:
            uri = f"{uri_base}/byTag/{tag_id}"
        else:
            logger.error("No filter criteria specified! You must pass in an option!")
            return Response()

        response: Response = self.session.get(uri)

        if page:
            del self.session.headers["page"]

        return response

    def stacks_get_summary(self):
        """Use to retrieve a list of stacks."""
        response: Response = self.session.get(f"{self.api_endpoint}/stacks/getSummary")
        return response

    def stacks_get_summary_cost(self, provider_id: str):
        """Use to retrieve a list of stack costs."""
        response: Response = self.session.get(
            f"{self.api_endpoint}/stacks/getSummaryWithCost/{provider_id}"
        )
        return response

    def stacks_get_connectivity(
        self, stack_id: str = ""
    ) -> RiscStackConnectivityParent:
        """Use to retrieve a list of connected stacks."""
        response: Response = self.session.get(
            f"{self.api_endpoint}/stacks/getConnectivity/{stack_id}"
        )
        if response.status_code != 200:
            return RiscStackConnectivityParent(response=response)

        connectivity_data = response.json()
        return RiscStackConnectivityParent(response=response, **connectivity_data)

    def stacks_get_device_connectivity(
        self, stack_id: int, connectivity_type: str = "internal", page: int = 0
    ) -> RiscDeviceConnectivityParent:
        """Use to retrieve a list of connected stacks."""
        if connectivity_type.lower() not in ["internal", "external"]:
            return RiscDeviceConnectivityParent()

        if page:
            self.session.headers.update({"page": str(page)})

        base_uri = f"{self.api_endpoint}/stacks/get{connectivity_type.title()}DeviceConnectivity"
        uri = f"{base_uri}/{'paginated/' if page else ''}{stack_id}"

        response: Response = self.session.get(uri)

        if page:
            del self.session.headers["page"]

        if response.status_code != 200:
            return RiscDeviceConnectivityParent(response=response)

        connectivity_data = response.json()
        return RiscDeviceConnectivityParent(response=response, **connectivity_data)

    def iaas_get_providers(self):
        """Use to retrieve a list of IaaS providers."""
        response: Response = self.session.get(f"{self.api_endpoint}/iaas/getProviders")
        return response

    def iaas_pricing(self, payload: Dict[str, str]):
        """Use to retrieve a list of IaaS pricing."""
        response: Response = self.session.post(
            f"{self.api_endpoint}/iaas/pricing", json=payload
        )
        return response

    def tags_get_tags(self, payload: Dict[str, str]):
        """Use to retrieve a list of IaaS providers."""
        response: Response = self.session.get(
            f"{self.api_endpoint}/tags/getTags", json=payload
        )
        return response

    def tags_add_tags(self, payload: Dict[str, str]):
        """Use to retrieve a list of IaaS providers."""
        response: Response = self.session.post(
            f"{self.api_endpoint}/tags/addTags", json=payload
        )
        return response

    def assets_search(self, search: str = ""):
        """Get RISC assessment data."""
        response: Response = self.session.get(
            f"{self.api_endpoint}/assets/search/{search}"
        )
        if response.status_code != 200:
            logger.error("Failed to retrieve asset data - Asset: (%s)!" % search)
            return {}
        return response

    def ucel_get_checks(self, device_id: str = ""):
        """Use to retrieve a list of checks that have been run against devices."""
        response: Response = self.session.get(
            f"{self.api_endpoint}/ucel/getChecks{'/' + device_id if device_id else ''}"
        )
        return response

    def ucel_get_assets(self, check_id: str = ""):
        """Use to retrieve data on the device(s) by check."""
        response: Response = self.session.get(
            f"{self.api_endpoint}/ucel/getAssets/{check_id}"
        )
        return response

    def ucel_get_assets_paginated(self, check_id: str = "", page: int = 1):
        """Use to retrieve data on the device(s) by check."""
        self.session.headers.update({"page": str(page)})
        response: Response = self.session.get(
            f"{self.api_endpoint}/ucel/getAssets/paginated/{check_id}"
        )
        del self.session.headers["page"]
        return response

    def stacks_get_listeners(self, stack_id: int):
        """Use to retrieve a list of listeners in a stack."""
        response: Response = self.session.get(
            f"{self.api_endpoint}/stacks/getListeners/{stack_id}"
        )
        return response

    def get_swagger(self):
        """Fetch the swagger API configuration file."""
        swagger_resource: Response = self.session.get(
            "https://api.riscnetworks.com/docs/_/resource_list.json"
        )
        return swagger_resource

    def get_server(self, search: str = "", compare: str = "hostname") -> Dict[str, Any]:
        """Sift through the asset search response and only return the relevant host.

        Args:
            search (str): The value to search for against the assets search API endpoint.
            compare (str): The key to be used to match the provided search value with.
                For example, this can be changed from hostname to identifying_ip.
                Defaults to: hostname.

        Returns:
            dict: The device asset object, as returned from the RISC API.

        """
        response: Response = self.assets_search(search=search)
        if response.status_code != 200:
            logger.error("Failure fetching host data!")
            return {}

        host_data = response.json().get("assets", [])
        try:
            return_data = next(
                item
                for item in host_data
                if item["data"][compare].lower() == search.lower()
            )
        except (KeyError, IndexError):
            logger.error("Unable to find the specified server: (%s)!" % search)
            return {}
        return return_data

    def get_application_ips(
        self, application: str, identifying_ips_only: bool = True
    ) -> List[Any]:
        """Get all associated IP addresses for the provided application stack.

        Args:
            application (str): The application stack name, as identified in RISC.
            identifying_ips_only (bool): Whether or not to return only the identifying_ip value
                for each asset found within the application stack. If set to: False, this method
                will iterate through all ips in the device data object. Defaults to: True.

        Returns:
            list of str: The list of device IP addresses present in the application stack.

        """
        ip_addresses = []
        stack_id = self.lookup_stack_id(application)
        if not stack_id:
            return []
        stack_assets = self.assets_get_assets(stack_id=stack_id)
        if stack_assets.status_code != 200:
            logger.error(
                "Failed to retrieve application stack assets for: (%s)" % application
            )
            return []

        for asset in stack_assets.json().get("assets", []):
            asset_data = asset.get("data", {})
            if identifying_ips_only:
                # Handle dict and list types differently, because this API has no standard responses or models...
                if isinstance(asset_data, dict):
                    ip = asset_data.get("identifying_ip", "")
                elif isinstance(asset_data, list):
                    ip = next(
                        item["identifying_ip"]
                        for item in asset_data
                        if "identifying_ip" in item
                    )
                if ip and ip != "NULL":
                    ip_addresses.append(ip)
            else:
                if isinstance(asset_data, dict):
                    found_ips = asset_data.get("ips", [])
                elif isinstance(asset_data, list):
                    found_ips = next(
                        item["ips"] for item in asset_data if "ips" in item
                    )
                if found_ips:
                    ips = [
                        found_ip["ip"]
                        for found_ip in found_ips
                        if found_ip["ip"] != "NULL"
                    ]
                    ip_addresses += ips
        ip_addresses = sorted(list(set(ip_addresses)))
        return ip_addresses

    def get_disks(
        self, search: str, fudge_factor: float = 1.5, only_local_disks: bool = True
    ) -> Dict[str, Any]:
        """Get disk data for a specific asset.

        Args:
            search (str): The asset hostname to lookup.
            fudge_factor (float): The value multiplier to be used when calculating volume size
                to be returned.  This can be used to estimate a buffer over current device usage.
                Defaults to: 1.0 to represent the existing disk usage.
            only_local_disks (bool): Whether or not to reduce the response to include only local disks.
                Defaults to: True.

        Returns:
            dict: The mapping of disk data provided by RISC and additional drive usage data.

        """
        data = {}
        asset = self.get_server(search=search)
        disks = asset.get("data", {}).get("disks_logical", [])
        for disk in disks:
            # If the disk isn't a local disk, i.e. Compact, ignore it.
            if only_local_disks and disk["disk_type"] != "Local Disk":
                continue

            usage_kwargs = {
                "total_size": disk["disk_size_bytes"],
                "free_size": disk["disk_free_space_bytes"],
            }
            drive_letter = disk["disk_name"].rstrip(":")

            # Provide disk usage in KB, MB, GB, or TB based on size.
            disk["usage"] = handle_disk_sizing(fudge_factor=1.0, **usage_kwargs)
            # Provide disk sizing estimations based on utilized vs free space on disks multipled by fudge factor.
            disk["drive_sizing_estimations"] = handle_disk_sizing(
                fudge_factor=fudge_factor, **usage_kwargs
            )

            data[drive_letter] = disk
        return data


def main() -> None:
    """Define the main entry method for the CLI."""
    # Run RISC via the pipeline object for easy CLI access.
    fire.Fire(RISC)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""Define the RISC utilities."""
import math
from typing import Any, Dict

from risc.__version__ import __version__ as risc_version


def get_user_agent(user_agent: str = "risc-python") -> str:
    """Get the current module version."""
    user_agent_str: str = f"{user_agent}/{risc_version}"
    return user_agent_str


def roundup(x: float) -> int:
    """Round the provided float up to the nearest tens."""
    return int(math.ceil(x / 10.0)) * 10


def format_bytes(size) -> Dict[str, Any]:
    """Format bytes to KB, MB, GB, and TB."""
    # 2**10 = 1024
    power = 2 ** 10
    n = 0
    power_labels = {0: "", 1: "K", 2: "M", 3: "G", 4: "T"}
    while size > power:
        size /= power
        n += 1
    return {"size": roundup(size), "label": f"{power_labels[n]}B"}


def handle_disk_sizing(
    total_size: str, free_size: str, fudge_factor: float = 1.5
) -> Dict[str, Any]:
    """Determine disk sizing based on the provided fudge factor and utilized space."""
    free = int(free_size)
    total = int(total_size)
    used = total - free
    proposed_size = used * fudge_factor
    recommended = proposed_size if proposed_size <= total and proposed_size != 0 else total
    return format_bytes(recommended)



# -*- coding: utf-8 -*-
"""Define the RISC models module."""
import logging
import uuid as _uuid
from dataclasses import dataclass, field
from typing import Any, Dict, List

import pandas as pd
from requests.models import Response
from requests.sessions import Session

logger = logging.getLogger(__name__)


@dataclass
class AbstractRiscModel:
    """Abstract resource model for all RISC models."""

    uuid: _uuid.UUID = _uuid.uuid4()

    def __str__(self):
        """Define the string representation of the abstract RISC model."""
        return f"<{self.__class__.__name__}: {self.uuid!s}>"

    @property
    def to_dict_items(self):
        """Get the dictionary items of the model's attributes."""
        return self.__dict__.items()

    @property
    def to_dict(self):
        """Get the dictionary representation of the model's attributes."""
        items = self.to_dict_items
        return dict(items)

    def to_list_factory(self, class_type: Any, factory_objects: List[Any]) -> List[Any]:
        """Handle the casting of lists to a list of the provided object type."""
        factories_data: List[Any] = []

        if not factory_objects:
            return factories_data

        if isinstance(factory_objects[0], class_type):
            factories_data = factory_objects
        else:
            factories_data = [class_type(**item) for item in factory_objects]

        return factories_data

    def to_obj_factory(self, class_type: Any, factory_object: Any) -> Any:
        """Handle the casting of an object to the provided object type."""
        if not factory_object:
            return class_type()

        if isinstance(factory_object, class_type):
            return factory_object
        return class_type(**factory_object)


@dataclass
class RiscResponse(AbstractRiscModel):
    """Abstract resource model for all RISC models."""

    response: Response = Response()
    session: Session = Session()
    items: List[Any] = field(default_factory=list)
    return_status: str = ""
    return_status_detail: str = ""
    json: Dict[str, str] = field(default_factory=dict)
    page: int = 0

    # def __repr__(self):
    #     """Define the representation of a RISC response."""
    #     items = ", ".join(f"{item!s}" for item in self.items)
    #     return f"<{self.__class__.__name__}: ({items})>"


@dataclass
class RiscResourceModel(AbstractRiscModel):
    """Abstract resource model for all RISC models."""

    response: Response = Response()


@dataclass
class RiscAssessment(RiscResourceModel):
    """Define the Assessment resource model schema."""

    address: str = ""
    appliance_public_ip: str = ""
    assessment_code: str = ""
    assessment_stage_description: str = ""
    assessment_stage_name: str = ""
    city: str = ""
    company_name: str = ""
    country: str = ""
    end_date: str = ""
    start_date: str = ""
    state: str = ""
    zip: str = ""
    json: Dict[str, str] = field(default_factory=dict)

    def __str__(self):
        """Define the string representation of the abstract RISC model."""
        return f"<{self.__class__.__name__}: {self.assessment_code}>"

    @property
    def is_demo(self):
        """Determine whether or not the Assessment is a demo."""
        return (
            self.company_name == "Customer Sandbox"
            and self.state == "Kentucky"
            and self.zip == "12345"
            and self.appliance_public_ip == "unknown"
        )


@dataclass
class RiscAssessments(RiscResponse):
    """Define the Assessments resource model schema."""

    assessments: List[Any] = field(default_factory=list)

    def __post_init__(self):
        """Handle post initialization steps."""
        self.assessments: List[RiscAssessment] = self.to_list_factory(
            class_type=RiscAssessment, factory_objects=self.assessments
        )


@dataclass
class RiscStacks(RiscResponse):
    """Define the Stacks resource model schema."""

    stacks: List[Any] = field(default_factory=list)

    def __post_init__(self):
        """Handle post initialization steps."""
        self.stacks: List[RiscStack] = self.to_list_factory(
            class_type=RiscStack, factory_objects=self.stacks
        )


@dataclass
class RiscStackConnectivity(RiscResourceModel):
    """Define the Connectivity resource model schema."""

    avg_duration_per_connection: float = 0.0
    avg_kbps_per_connection: float = 0.0
    avg_rtt_per_connection: float = 0.0
    connections_seen_via_netstat: int = 0
    dest_location: int = 0
    dest_location_name: str = ""
    max_kbps_per_connection: float = 0.0
    max_rtt_per_connection: float = 0.0
    min_rtt_per_connection: float = 0.0
    number_of_distinct_protocols: int = 0
    number_of_distinct_source_ip_dest_ip_pairs: int = 0
    source_location_name: str = ""
    source_locationid: int = 0
    total_bytes: int = 0
    total_duration_per_connection: float = 0.0
    total_flows: int = 0


@dataclass
class RiscStackConnectivityParent(RiscResourceModel):
    """Define the parent Stack Connectivity resource model schema."""

    connectivity: List[Any] = field(default_factory=list)
    returnStatus: str = ""
    returnStatusDetail: str = ""

    def __post_init__(self):
        """Handle post initialization steps."""
        self.connectivity = self.to_list_factory(
            class_type=RiscStackConnectivity, factory_objects=self.connectivity
        )

    @property
    def dataframe(self):
        """Handle converting the connectivity list of objects to a pandas DataFrame."""
        connectivity_dict: List[Dict[str, Any]] = self.response.json().get(
            "connectivity", []
        )
        if not connectivity_dict:
            return pd.DataFrame()
        return pd.DataFrame.from_dict(connectivity_dict)


@dataclass
class RiscDeviceConnectivity(RiscResourceModel):
    """Define the Device Connectivity resource model schema."""

    avg_duration: float = 0.0
    avg_kbps: float = 0.0
    avg_rtt: float = 0.0
    dest_application: str = ""
    dest_application_context: str = ""
    dest_application_instance: str = ""
    dest_bytes: int = 0
    dest_deviceid: int = 0
    dest_ip: str = ""
    dest_packet_count: int = 0
    dest_packet_loss: int = 0
    dest_port: int = 0
    dest_process: str = ""
    max_kbps: float = 0.0
    max_rtt: float = 0.0
    min_rtt: float = 0.0
    netstat_count: int = 0
    source_application: str = ""
    source_application_context: str = ""
    source_application_instance: str = ""
    source_bytes: int = 0
    source_deviceid: int = 0
    source_packet_count: int = 0
    source_packet_loss: int = 0
    source_process: str = ""
    src_ip: str = ""
    total_bytes: int = 0
    total_duration: float = 0.0
    total_packets: int = 0


@dataclass
class RiscDeviceConnectivityParent(RiscResourceModel):
    """Define the parent Device Connectivity resource model schema."""

    connectivity: List[Any] = field(default_factory=list)
    returnStatus: str = ""
    returnStatusDetail: str = ""

    def __post_init__(self):
        """Handle post initialization steps."""
        self.connectivity = self.to_list_factory(
            class_type=RiscDeviceConnectivity, factory_objects=self.connectivity
        )

    @property
    def dataframe(self):
        """Handle converting the connectivity list of objects to a pandas DataFrame."""
        connectivity_dict: List[Dict[str, Any]] = self.response.json().get(
            "connectivity", []
        )
        if not connectivity_dict:
            return pd.DataFrame()
        return pd.DataFrame.from_dict(connectivity_dict)


@dataclass
class RiscTag(RiscResourceModel):
    """Define the Tag resource model schema."""

    tagid: int = 0
    tagkey: str = ""
    tagvalue: str = ""


@dataclass
class RiscStack(RiscResourceModel):
    """Define the Stack resource model schema."""

    confirmed: str = ""
    confirmedby: str = "not set"
    licensed: int = 0
    num_members_with_failed_checks: int = 0
    num_stack_members: int = 0
    stack_name: str = ""
    stackid: int = 0
    tags: List[Any] = field(default_factory=list)

    def __post_init__(self):
        """Handle post initialization steps."""
        self.tags: List[RiscTag] = self.to_list_factory(
            class_type=RiscTag, factory_objects=self.tags
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Define the RISC module setup.

Note: To use the 'upload' functionality of this file, you must:

`$ pipenv install twine --dev`

"""

import io
import os
import sys
from shutil import rmtree
from typing import Dict, List

from setuptools import Command, find_packages, setup

# Package meta-data.
NAME: str = "risc"
DESCRIPTION: str = "RISC Python Client and CLI"
URL: str = "https://github.com/2ndWatch/risc-python"
EMAIL: str = "mbeacom@2ndwatch.com"
AUTHOR: str = "Mark Beacom"
REQUIRES_PYTHON: str = ">=3.6.0"
VERSION: str = os.environ.get("RISC_CLIENT_VERSION", "")

REQUIRED: List[str] = ["requests", "fire"]
EXTRAS: Dict[str, List[str]] = {
    "test": [
        "coverage",
        "isort",
        "pytest",
        "pytest-cov",
        "pytest-isort",
        "pytest-sugar",
        "black",
        "pydocstyle",
        "pycodestyle",
        "bandit",
        "mypy",
        "twine",
    ]
}

here = os.path.abspath(os.path.dirname(__file__))

try:
    with io.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
        long_description: str = "\n" + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# Load the package's __version__.py module as a dictionary.

about: Dict[str, str] = {}
if not VERSION:
    project_slug: str = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here, project_slug, "__version__.py")) as f:
        exec(f.read(), about)
else:
    about["__version__"] = VERSION


class UploadCommand(Command):
    """Support setup.py upload."""

    description: str = "Build and publish the package."
    user_options: List[str] = []

    @staticmethod
    def status(s):
        """Print things in bold."""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(here, "dist"))
        except OSError:
            pass

        self.status("Building Source and Wheel distribution…")
        sys_exec: str = sys.executable
        os.system(f"{sys_exec} setup.py sdist bdist_wheel")

        self.status("Uploading the package to PyPI via Twine…")
        os.system("twine upload dist/*")

        self.status("Pushing git tags…")
        vers: str = about["__version__"]
        os.system(f"git tag v{vers}")
        os.system("git push --tags")

        sys.exit()


# Where the magic happens:
setup(
    name=NAME,
    version=about["__version__"],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    entry_points={"console_scripts": ["risc=risc.main:main"]},
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: MacOS",
        "Operating System :: MacOS :: MacOS 9",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft",
        "Operating System :: Microsoft :: MS-DOS",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Operating System :: Microsoft :: Windows :: Windows 8.1",
        "Operating System :: Microsoft :: Windows :: Windows 8",
        "Operating System :: Microsoft :: Windows :: Windows 7",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Internet",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Session",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: User Interfaces",
        "Topic :: Utilities",
    ],
    # $ setup.py publish support.
    cmdclass={"upload": UploadCommand},
)
